"""
contains all the questions that need to be asked by the games
"""
from prompt_toolkit.validation import Validator, ValidationError
import regex
import csv

from pathlib import Path


def read_picked_slots() -> list:
    """
    This reads the picked_slots.csv and gets a list of picked slots

    :return: a list of picked slots
    """
    with open(Path.home() / '.gamehub/picked_slots.csv', newline='') as picked_slots_csv:
        picked_slots_rows = csv.reader(picked_slots_csv, delimiter=' ', quotechar='|')
        picked_slots = []
        for row in picked_slots_rows:
            picked_slots.append(row[0])
    return picked_slots


class IntValidator(Validator):
    """
    thing
    """

    def validate(self, document) -> None:
        """
        validates the input
        """

        not_ok = regex.search('[^0-9]', document.text)
        if not_ok:
            raise ValidationError(
                message='Please enter a int',
                cursor_position=len(document.text)
            )


class OneToNineValidator(Validator):
    """
    thing
    """

    def validate(self, document) -> None:
        """
        validates the input
        """
        has_already_been_picked = False
        picked_slots = read_picked_slots()
        for i in range(picked_slots.__len__()):
            if str(picked_slots[i]) in document.text:
                has_already_been_picked = True
        is_not_from_1_to_9 = not regex.match('^[0-9]{1}$', document.text)
        if is_not_from_1_to_9 or has_already_been_picked:
            raise ValidationError(
                message="Please enter a int from one to nine that hasn't been picked yet",
                cursor_position=len(document.text)
            )


class StringValidator(Validator):
    """
    thing
    """

    def validate(self, document) -> None:
        """
        validates the input
        """

        ok = regex.match('^[a-zA-Z]{1,11}$', document.text)
        if not ok:
            raise ValidationError(
                message='Please enter a string that is less than 10 characters',
                cursor_position=len(document.text)
            )


tic_questions = [
    # question 1 - Tutorial
    {
        'type': 'confirm',
        'message': 'Do you want to see the tutorial?',
        'name': 'tutorial',
        'default': True
    },
    # question 2 - Difficulty level
    {
        'type': 'list',
        'name': 'difficulty',
        'message': 'What difficulty level do you want?',
        'choices': ['Easy', 'Medium', 'Hard', 'Custom'],
    },
    # question 3 - freq of lose
    {
        'type': 'input',
        'name': 'freq_lose',
        'message': 'frequency of losing moves:',
        'validate': IntValidator
    },
    # question 4 - freq of draw
    {
        'type': 'input',
        'name': 'freq_draw',
        'message': 'frequency of drawing moves:',
        'validate': IntValidator
    },
    # question 5 - freq of win
    {
        'type': 'input',
        'name': 'freq_win',
        'message': 'frequency of winning moves:',
        'validate': IntValidator
    },
    # question 6 - confirm frequencies
    {
        'type': 'confirm',
        'name': 'confirm_choices',
        'message': 'Are you sure about these choices?',
    },
    # question 7 - human or ai
    {
        'type': 'confirm',
        'name': 'human_or_ai',
        'message': 'Do you want to play against the AI?',
    },
    # question 8 - pick a slot
    {
        'type': 'input',
        'name': 'picked_slot',
        'message': 'Pick a slot:',
        'validate': OneToNineValidator
    },
]

hangman_questions = [
    {
        'type': 'input',
        'message': 'Type your guess:',
        'name': 'hangman_guess',
        'validate': StringValidator
    },
]

rps_questions = [
    {
        'type': 'list',
        'name': 'rps',
        'message': 'Pick rock paper or scissors: ',
        'choices': ['Rock', 'Paper', 'Scissors'],
    },
]



"""
this is the hangman game
"""
import random

from PyInquirer import prompt
from examples import custom_style_1

from game_hub.utils.file.path import find_file
from game_hub.utils.logging import MyLogger
from game_hub.questions import hangman_questions

logger = MyLogger.logging.getLogger('gamehub.games.hangMan')


def randomise(file_path=None) -> str:
    """
    returns a random word from the hangManWords.txt file
    :return: random word
    """
    if file_path is None:
        file_name = 'hangManWords.txt'
        file_path = find_file(file_name)
        if file_path is None:
            file_path = find_file(file_name, '../')
    with open(file_path, 'r') as hang_man_words:
        hang_man_words_string = hang_man_words.read()
    words = hang_man_words_string.split()
    random_word_picker = random.randint(0, len(words) - 1)
    return words[random_word_picker]


def hiding(word: str) -> str:
    """
    takes a word and replaces all characters with underscores
    :return: word with characters as underscores
    """
    hidden_word = ""
    counter = 0
    broken_up_random_word = list(word)
    number_of_letters_in_random_word: int = len(broken_up_random_word)
    while counter != number_of_letters_in_random_word:
        hidden_word += "_"
        counter += 1
    the_word = ' '.join(list(hidden_word))
    logger.info(the_word)
    return hidden_word


def hang_man_game(word: str, hidden_word: str, debug: str = None) -> str:
    """
    it loops while the player hasn't won or lost yet, and when it loops it asked the user for a letter, an if it's
    in the word then it replaces the space where the letter was originally in the hidden word and then prints it, if
    they get it wrong then the function takes off one life and prints incorrect then the hidden word again. if the
    user types in the full word or they guess letter by letter they win, but lose if they run out of lives
    :param debug: string to confirm if you are debugging, and if you are, then is it a test or manual debugging
    :param word: the word that the user has to guess
    :param hidden_word: the word but all chars are replaced with underscores
    """
    guess = ''
    debug_word_picker = 0
    hidden_word_chars = list(hidden_word)
    chances: int = 10
    word_chars = list(word)
    while word_chars != hidden_word_chars and chances > 0:
        if debug is not None:
            if debug == "test":
                guess = debug[debug_word_picker]
            elif debug == "manual":
                guess = input("enter 'guess': ")
        else:
            guess = prompt(hangman_questions[0], style=custom_style_1).get("hangman_guess")
        if guess == word:
            break
        hidden_word_chars, is_guess_correct = replace_space_with_guess(guess, word_chars, hidden_word_chars)
        logger.info(" ".join(hidden_word_chars))
        if not is_guess_correct:
            chances -= 1
            logger.info("incorrect!")
            if chances > 1:
                logger.info("You have " + str(chances) + " lives left")
            elif chances == 1:
                logger.info("You have " + str(chances) + " life left")
        debug_word_picker += 1
    if chances == 0:
        return f"You failed,\nThe word was {word}"
    else:
        return "Congrats, you won!"


def replace_space_with_guess(guess: str, list_word: list, list_hidden_word: list) -> list and bool:
    """
    replaces underscore with guess if the guess is correct
    :return: None
    """
    was_the_guess_correct = False
    for i in range(len(list_word)):
        if guess == list_word[i]:
            was_the_guess_correct = True
            list_hidden_word[i] = guess
    return list_hidden_word, was_the_guess_correct


def game() -> None:
    """
    plays the hangman game
    :return:
    """
    logger.info("the hang man game has been picked")
    random_word = randomise()
    end_message = hang_man_game(random_word, hiding(random_word))
    logger.info(end_message)


if __name__ == '__main__':
    game()
